@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: cpp.c,v 1.179 2010/09/18 11:56:50 marcus Exp $
+|| $Id: 6dd5904a0976901c827ad0142bc83134d6ffa82d $
 */
 
 #include "global.h"
@@ -1816,6 +1816,7 @@ void f_cpp(INT32 args)
     if(charset_sv->type == T_STRING) {
       charset = charset_sv->u.string;
       push_string(data);
+      this.data = data = NULL;
       ref_push_string(charset);
       if (!safe_apply_handler ("decode_charset", this.handler,
 			       this.compat_handler, 2, BIT_STRING)) {
@@ -1823,7 +1824,6 @@ void f_cpp(INT32 args)
 			      charset);
 	Pike_error("Unknown charset.\n");
       }
-      free(data);
       this.data = data = sp[-1].u.string;
       sp--;
       dmalloc_touch_svalue(sp);
