@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: f9551098b61e4b8bb171ed1a47d9873f0e30b103 $
+|| $Id: 40d72e38f38fd4dd240a60d9b53f652bf074c41a $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -783,6 +783,28 @@ static void f_error_index(INT32 args)
   }
 }
 
+static void f_error__sizeof(INT32 args)
+{
+  pop_n_elems(args);
+  push_int(2);
+}
+
+static void f_error__indices(INT32 args)
+{
+  pop_n_elems(args);
+  push_int(0);
+  push_int(1);
+  f_aggregate(2);
+}
+
+static void f_error__values(INT32 args)
+{
+  pop_n_elems(args);
+  apply_current(generic_err_message_fun, 0);
+  apply_current(generic_err_backtrace_fun, 0);
+  f_aggregate(2);
+}
+
 /*! @decl string describe()
  *!
  *! Return a readable error report that includes the backtrace.
