@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: fdlib.c,v 1.90 2009/05/28 22:43:57 grubba Exp $
+|| $Id: c8a4d9ea70506d8281d60d2908ad78d48fdfcbbc $
 */
 
 #include "global.h"
@@ -33,6 +33,7 @@ int fd_type[MAX_OPEN_FILEDESCRIPTORS];
 int first_free_handle;
 
 /* #define FD_DEBUG */
+/* #define FD_STAT_DEBUG */
 
 #ifdef FD_DEBUG
 #define FDDEBUG(X) X
@@ -40,6 +41,12 @@ int first_free_handle;
 #define FDDEBUG(X)
 #endif
 
+#ifdef FD_STAT_DEBUG
+#define STATDEBUG(X) X
+#else
+#define STATDEBUG(X) do {} while (0)
+#endif
+
 /* _dosmaperr is internal but still exported in the dll interface. */
 __declspec(dllimport) void __cdecl _dosmaperr(unsigned long);
 
@@ -452,7 +459,9 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
     drive = toupper(*file) - 'A';
   else
     drive = -1;
-  
+
+  STATDEBUG (fprintf (stderr, "fd_stat %s drive %d\n", file, drive));
+
   /* get info for file */
   hFind = FindFirstFile(file, &findbuf);
 
@@ -461,14 +470,24 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
     char abspath[_MAX_PATH + 1];
     UINT drive_type;
 
-    if (!strpbrk(file, "./\\") ||
-	!_fullpath( abspath, file, _MAX_PATH ) ||
-	/* root dir. ('C:\') or UNC root dir. ('\\server\share\') */
-	(strlen (abspath) > 3 && !IsUncRoot (abspath))) {
+    STATDEBUG (fprintf (stderr, "check root dir\n"));
+
+    if (!strpbrk(file, ":./\\")) {
+      STATDEBUG (fprintf (stderr, "no path separators\n"));
       errno = ENOENT;
       return -1;
     }
 
+    if (!_fullpath( abspath, file, _MAX_PATH ) ||
+	/* Neither root dir ('C:\') nor UNC root dir ('\\server\share\'). */
+	(strlen (abspath) > 3 && !IsUncRoot (abspath))) {
+      STATDEBUG (fprintf (stderr, "not a root %s\n", abspath));
+      errno = ENOENT;
+      return -1;
+    }
+
+    STATDEBUG (fprintf (stderr, "abspath: %s\n", abspath));
+
     l = strlen (abspath);
     if (!ISSEPARATOR (abspath[l - 1])) {
       /* Ensure there's a slash at the end or else GetDriveType
@@ -479,10 +498,13 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
 
     drive_type = GetDriveType (abspath);
     if (drive_type == DRIVE_UNKNOWN || drive_type == DRIVE_NO_ROOT_DIR) {
+      STATDEBUG (fprintf (stderr, "invalid drive type: %u\n", drive_type));
       errno = ENOENT;
       return -1;
     }
 
+    STATDEBUG (fprintf (stderr, "faking root stat\n"));
+
     /* Root directories (e.g. C:\ and \\server\share\) are faked */
     findbuf.dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY;
     findbuf.nFileSizeHigh = 0;
@@ -508,7 +530,12 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
   }
 
   else {
-    char fstype[5]; /* Room for "FAT" and anything longer that begins with "FAT". */
+    char fstype[50];
+    /* Really only need room in this buffer for "FAT" and anything
+     * longer that begins with "FAT", but since GetVolumeInformation
+     * has shown to trig unreliable error codes for a too short buffer
+     * (see below), we allocate ample space. */
+
     BOOL res;
 
     if (drive >= 0) {
@@ -521,13 +548,22 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
     else
       res = GetVolumeInformation (NULL, NULL, 0, NULL, NULL,NULL,
 				  (LPSTR)&fstype, sizeof (fstype));
-    if (!res &&
-	/* Get ERROR_MORE_DATA if the fstype buffer wasn't long
-	 * enough, so let's ignore it. */
-	GetLastError() != ERROR_MORE_DATA) {
-      set_errno_from_win32_error (GetLastError());
-      FindClose (hFind);
-      return -1;
+
+    STATDEBUG (fprintf (stderr, "found, vol info: %d, %s\n",
+			res, res ? fstype : "-"));
+
+    if (!res) {
+      unsigned long w32_err = GetLastError();
+      /* Get ERROR_MORE_DATA if the fstype buffer wasn't long enough,
+       * so let's ignore it. That error is also known to be reported
+       * as ERROR_BAD_LENGTH in Vista and 7. */
+      if (w32_err != ERROR_MORE_DATA && w32_err != ERROR_BAD_LENGTH) {
+	STATDEBUG (fprintf (stderr, "GetVolumeInformation failure: %d\n",
+			    w32_err));
+	set_errno_from_win32_error (w32_err);
+	FindClose (hFind);
+	return -1;
+      }
     }
 
     if (res && !strcmp (fstype, "FAT")) {
@@ -535,6 +571,7 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
 				       &findbuf.ftLastAccessTime,
 				       &findbuf.ftLastWriteTime,
 				       buf)) {
+	STATDEBUG (fprintf (stderr, "fat_filetimes_to_stattimes failed.\n"));
 	FindClose (hFind);
 	return -1;
       }
