@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 34e6ca7c44e3fd4e71e0d0a2c2ff61d75ba687fc $
+|| $Id: 7a8b9cf9fc93d321096264cf3c296313b29f3dff $
 */
 
 #ifndef GLOBAL_H
@@ -552,8 +552,12 @@ typedef struct p_wchar_p
 #define PMOD_PROTO
 #endif
 
+#if defined(USE_VALGRIND) && !defined(__CHECKER__)
+#define __CHECKER__
+#endif
+
 #ifndef DO_PIKE_CLEANUP
-#if defined(PURIFY) || defined(__CHECKER__) || defined(DEBUG_MALLOC)
+#if defined(PURIFY) || defined(__CHECKER__) || defined(DEBUG_MALLOC) || defined(USE_VALGRIND)
 #define DO_PIKE_CLEANUP
 #endif
 #endif
