@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: interpret.c,v 1.414 2010/01/21 15:39:58 grubba Exp $
+|| $Id: a7bac8b90abc1a60853711a01099733eb66de584 $
 */
 
 #include "global.h"
@@ -281,6 +281,8 @@ use_malloc:
   interpreter->accounted_time = 0;
 #endif
 
+  interpreter->trace_level = default_t_flag;
+
   return 0;	/* OK. */
 }
 
@@ -920,10 +922,10 @@ static INLINE void low_debug_instr_prologue (PIKE_INSTR_T instr)
     if(d_flag>1) DEBUG_CHECK_THREAD();
 #endif
 
-    Pike_sp[0].type=99; /* an invalid type */
-    Pike_sp[1].type=99;
-    Pike_sp[2].type=99;
-    Pike_sp[3].type=99;
+    INVALIDATE_SVAL(Pike_sp[0]);
+    INVALIDATE_SVAL(Pike_sp[1]);
+    INVALIDATE_SVAL(Pike_sp[2]);
+    INVALIDATE_SVAL(Pike_sp[3]);
       
     if(Pike_sp<Pike_interpreter.evaluator_stack ||
        Pike_mark_sp < Pike_interpreter.mark_stack || Pike_fp->locals>Pike_sp)
@@ -2126,7 +2128,8 @@ void unlink_previous_frame(void)
       Pike_interpreter.accounted_time - current->children_base;
     struct identifier *function =
       current->context->prog->identifiers + current->ident;
-    function->total_time += total_time;
+    if (!function->recur_depth)
+      function->total_time += total_time;
     total_time -= child_time;
     function->self_time += total_time;
     Pike_interpreter.accounted_time += total_time;
