@@ -1,4 +1,4 @@
-#line 1 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1 "/Users/hww3/pikebuild/src/interpret_functions.h"
 /*
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
@@ -293,7 +293,7 @@ OPCODE1_TAIL(F_MARK_AND_STRING, "mark & string", I_UPDATE_SP|I_UPDATE_M_SP, {
     print_return_value();
   };
 })
-#line 287 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 287 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE1(F_STRING, "string", I_UPDATE_SP, {
     copy_shared_string(Pike_sp->u.string,Pike_fp->context->prog->strings[arg1]);
     Pike_sp->type=PIKE_T_STRING;
@@ -301,7 +301,7 @@ OPCODE1(F_STRING, "string", I_UPDATE_SP, {
     Pike_sp++;
     print_return_value();
   })
-#line 294 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 294 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 
@@ -384,13 +384,13 @@ OPCODE1_TAIL(F_MARK_AND_GLOBAL, "mark & global", I_UPDATE_SP|I_UPDATE_M_SP, {
     print_return_value();
   };
 })
-#line 370 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 370 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE1(F_GLOBAL, "global", I_UPDATE_SP, {
     low_index_current_object_no_free(Pike_sp, arg1);
     Pike_sp++;
     print_return_value();
   })
-#line 375 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 375 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 OPCODE2_TAIL(F_MARK_AND_EXTERNAL, "mark & external", I_UPDATE_SP|I_UPDATE_M_SP, {
@@ -424,7 +424,7 @@ OPCODE2_TAIL(F_MARK_AND_EXTERNAL, "mark & external", I_UPDATE_SP|I_UPDATE_M_SP,
     print_return_value();
   };
 })
-#line 380 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 380 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE2(F_EXTERNAL,"external", I_UPDATE_SP, {
     LOCAL_VAR(struct external_variable_context loc);
 
@@ -452,7 +452,7 @@ OPCODE2(F_EXTERNAL,"external", I_UPDATE_SP, {
     }
     print_return_value();
   })
-#line 407 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 407 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 OPCODE2(F_EXTERNAL_LVALUE, "& external", I_UPDATE_SP, {
@@ -1137,18 +1137,18 @@ OPCODE0_TAIL(F_MARK2, "mark mark", I_UPDATE_M_SP, {
     };
   };
 })
-#line 1084 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1084 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE0_TAIL(F_SYNCH_MARK, "synch mark", I_UPDATE_M_SP, {
 
     {
       *(Pike_mark_sp++)=Pike_sp;
     };
   })
-#line 1086 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1086 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE0(F_MARK, "mark", I_UPDATE_M_SP, {
       *(Pike_mark_sp++)=Pike_sp;
     })
-#line 1090 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1090 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 OPCODE1(F_MARK_X, "mark Pike_sp-X", I_UPDATE_M_SP, {
@@ -1185,7 +1185,7 @@ OPCODE0_TAIL(F_CLEANUP_SYNCH_MARK, "cleanup synch mark", I_UPDATE_SP|I_UPDATE_M_
     }
   };
 })
-#line 1108 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1108 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE0(F_POP_SYNCH_MARK, "pop synch mark", I_UPDATE_SP|I_UPDATE_M_SP, {
     if (d_flag) {
       if (Pike_mark_sp <= Pike_interpreter.mark_stack) {
@@ -1203,7 +1203,7 @@ OPCODE0(F_POP_SYNCH_MARK, "pop synch mark", I_UPDATE_SP|I_UPDATE_M_SP, {
       }
     }
   })
-#line 1125 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1125 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 OPCODE0(F_CLEAR_STRING_SUBTYPE, "clear string subtype", 0, {
@@ -1238,7 +1238,7 @@ OPCODE2_BRANCH(F_BRANCH_IF_NOT_LOCAL_ARROW, "branch if !local->x", 0, {
     pop_stack();
   };
 })
-#line 1148 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1148 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE0_TAILBRANCH(F_BRANCH_WHEN_ZERO, "branch if zero", I_UPDATE_SP, {
     if(!UNSAFE_IS_ZERO(Pike_sp-1))
     {
@@ -1249,7 +1249,7 @@ OPCODE0_TAILBRANCH(F_BRANCH_WHEN_ZERO, "branch if zero", I_UPDATE_SP, {
     }
     pop_stack();
   })
-#line 1158 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 1158 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
       
@@ -2708,11 +2708,11 @@ OPCODE1_PTRJUMP(F_COND_RECUR, "recur if not overloaded", I_UPDATE_ALL, {
     DO_RECUR(0);
   };
 })
-#line 2612 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 2612 "/Users/hww3/pikebuild/src/interpret_functions.h"
 OPCODE0_TAILPTRJUMP(F_RECUR, "recur", I_UPDATE_ALL, {
     DO_RECUR(0);
   })
-#line 2615 "/Users/hww3/pike-build/src/interpret_functions.h"
+#line 2615 "/Users/hww3/pikebuild/src/interpret_functions.h"
 
 
 /* Ugly code duplication */
