@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: bd7787e34836b1f2156266697589243fb9e8eece $
+|| $Id: a7af4dee2d3e783b98a7df659562c9bb3d598ba8 $
 */
 
 /*
@@ -685,7 +685,7 @@ static void image_x_encode_pseudocolor_2byte(INT32 args,
 	   /* NOTE: The ntohs macro may evaluate it's argument more
 	    * than once. */
 	    unsigned short t = translate[*(s++)];
-	    b=ntohs(t)<<(32-vbpp); 
+	    b=(unsigned long)ntohs(t)<<(32-vbpp); 
 	    bp = bpp;
 	    while (bp>8-bit)
 	    {
@@ -704,7 +704,7 @@ static void image_x_encode_pseudocolor_2byte(INT32 args,
 	 x=img->xsize; 
 	 while (x--) 
 	 {
-	    b=(*(s++))<<(32-bpp); 
+	    b=(unsigned long)(*(s++))<<(32-bpp); 
 	    bp = bpp;
 	    while (bp>8-bit)
 	    {
