@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: msqlmod.c,v 1.31 2009/02/15 16:15:41 srb Exp $
+|| $Id: 74177bb151cbb484e8625e240fc81b20a9a4f22d $
 */
 
 /* All this code is pretty useless if we don't have a msql library...*/
@@ -281,8 +281,9 @@ static void msql_mod_create (INT32 args)
 	char *colon;
 
 	check_all_args("Msql->create",args,
-			BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,
-			BIT_STRING|BIT_VOID,0);
+		       BIT_STRING|BIT_VOID, BIT_STRING|BIT_VOID,
+		       BIT_STRING|BIT_VOID, BIT_STRING|BIT_VOID,
+		       BIT_MAPPING|BIT_INT|BIT_VOID, 0);
 
 	if (args)
 	  if (sp[-args].u.string->len)
