@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 7ae5ae97a9bb16ce63bbb2ac62913735eac8c780 $
+|| $Id: 2a438cfa5996355d77dd5b8446161cf2c23ea248 $
 */
 
 /*
@@ -480,13 +480,13 @@ static void pike_mysql_reconnect (int reconnect)
 
   MYSQL_ALLOW();
 
+#if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
+  STUPID_PORT_LOCK();
+#endif /* HAVE_MYSQL_PORT || HAVE_MYSQL_UNIX_PORT */
 #ifdef HAVE_MYSQL_REAL_CONNECT
   socket = mysql_real_connect(mysql, host, user, password,
                               NULL, port, portptr, options);
 #else
-#if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
-  STUPID_PORT_LOCK();
-#endif /* HAVE_MYSQL_PORT || HAVE_MYSQL_UNIX_PORT */
 
 #ifdef HAVE_MYSQL_PORT
   if (port) {
@@ -514,10 +514,10 @@ static void pike_mysql_reconnect (int reconnect)
   }
 #endif /* HAVE_MYSQL_UNIX_PORT */
 
+#endif /* HAVE_MYSQL_REAL_CONNECT */
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
   STUPID_PORT_UNLOCK();
 #endif /* HAVE_MYSQL_PORT || MAVE_MYSQL_UNIX_PORT*/
-#endif /* HAVE_MYSQL_REAL_CONNECT */
 
   MYSQL_DISALLOW();
 
@@ -1820,6 +1820,25 @@ static void f__can_send_as_latin1 (INT32 args)
 /*! @endclass
  */
 
+/*! @decl string client_info()
+ *!
+ *! Get some information about the Mysql-server client library.
+ *!
+ *! @seealso
+ *!   @[mysql()->statistics()], @[mysql()->server_info()],
+ *!   @[mysql()->protocol_info()], @[mysql()->info()]
+ */
+static void f_client_info(INT32 args)
+{
+  pop_n_elems(args);
+
+#ifndef MYSQL_COMPILATION_COMMENT
+#define MYSQL_COMPILATION_COMMENT "MySQL (Copyright Abandoned)"
+#endif
+
+  push_text(MYSQL_COMPILATION_COMMENT "/" MYSQL_SERVER_VERSION);
+}
+
 /*! @endmodule
  */
 
@@ -1923,6 +1942,9 @@ PIKE_MODULE_INIT
   mysql_program = end_program();
   add_program_constant("mysql", mysql_program, 0);
 
+  /* function(void:string) */
+  ADD_FUNCTION("client_info", f_client_info,tFunc(tVoid,tStr), ID_PUBLIC);
+
 #ifdef HAVE_MYSQL_PORT
   STUPID_PORT_INIT();
 #endif /* HAVE_MYSQL_PORT */
