@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 034336a4200298360a9f872f37c4f234476f1639 $
+|| $Id: 0136db1ac90cadf5e345461915cbda2441a00da0 $
 */
 
 /*
@@ -280,9 +280,11 @@ static void odbc_fix_fields(void)
       break;
     case SQL_DATE:
       push_text("date");
+      odbc_field_sizes[i] = 32;
       break;
     case SQL_TIMESTAMP:
       push_text("time");
+      odbc_field_sizes[i] = 32;
       break;
     case SQL_LONGVARCHAR:
 #ifdef SQL_WLONGVARCHAR
@@ -571,6 +573,7 @@ static void f_fetch_row(INT32 args)
 {
   SQLHSTMT hstmt = PIKE_ODBC_RES->hstmt;
   int i;
+  unsigned int old_tds_kludge = PIKE_ODBC_RES->odbc->flags & PIKE_ODBC_OLD_TDS_KLUDGE;
   RETCODE code;
  
   pop_n_elems(args);
@@ -613,9 +616,7 @@ static void f_fetch_row(INT32 args)
        * preferred, since less buffer space is wasted.
        */
 
-#ifdef __NT__
-      if (!len) {
-#endif /* __NT__ */
+      if (old_tds_kludge || !len) {
 
 	ODBC_ALLOW();
 
@@ -660,9 +661,7 @@ static void f_fetch_row(INT32 args)
 	PIKE_ODBC_RES->field_info[i].type = field_type;
 #endif
 
-#ifdef __NT__
       }
-#endif /* __NT__ */
 
 #ifdef ODBC_DEBUG
       fprintf(stderr, "ODBC:fetch_row(): Field %d: "
