@@ -1,13 +1,14 @@
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: 88ad1113ee54f1b016801fabdc041a2843add83b $");
+RCSID("$Id: 773e3fb1cd199102632ced91ef7d28189278f5a9 $");
 #include "interpret.h"
 #include "program.h"
 #include "object.h"
 #include "array.h"
 #include "module_support.h"
 #include "fsort.h"
+#include "bignum.h"
 
 #include "config.h"
 
@@ -43,7 +44,8 @@ int wf_blob_next( Blob *b )
     }
     ref_push_string( b->word );
     push_int( b->docid );
-    apply_svalue( b->feed, 2 );
+    push_longest( (LONGEST) b );
+    apply_svalue( b->feed, 3 );
     if( sp[-1].type != T_STRING )
     {
       b->eof = 1;
@@ -65,7 +67,8 @@ int wf_blob_next( Blob *b )
       }
       ref_push_string( b->word );
       push_int( b->docid );
-      apply_svalue( b->feed, 2 );
+      push_longest( (LONGEST) b );
+      apply_svalue( b->feed, 3 );
       if( sp[-1].type != T_STRING )
       {
 	b->eof = 1;
