@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: file.h,v 1.41 2010/02/24 13:14:59 grubba Exp $
+|| $Id: adbb8ad95d15556de43927b90533315ad81110af $
 */
 
 #ifndef FILE_H
@@ -41,6 +41,20 @@ struct my_file
   short flags;
   int my_errno;
 
+#ifdef HAVE_PIKE_SEND_FD
+  int *fd_info;
+  /* Info about fds pending to be sent.
+   *   If non-NULL the first element is the array size,
+   *   and the second is the number of fds pending to be
+   *   sent. Elements three and onwards are fds to send.
+   *
+   *   Note that to avoid races between the call to
+   *   send_fd() and the call to write(), these fds
+   *   are dup(2)'ed in send_fd() and close(2)'ed after
+   *   sending in write() (or close() or destroy()).
+   */
+#endif
+
 #if defined(HAVE_FD_FLOCK) || defined(HAVE_FD_LOCKF)
   struct object *key;
 #endif
@@ -89,7 +103,12 @@ struct pike_sendfile
 extern struct program *file_program;
 extern struct program *file_ref_program;
 
-extern int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port, int udp);
+/* Note: Implemented in ../system/system.c! */
+extern int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service,
+			 INT_TYPE port, int inet_flags);
+#define PIKE_INET_FLAG_UDP	1
+#define PIKE_INET_FLAG_IPV6	2
+#define PIKE_INET_FLAG_NB	4
 
 #ifdef _REENTRANT
 void low_do_sendfile(struct pike_sendfile *);
