@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: udp.c,v 1.84 2010/07/27 16:46:05 mast Exp $
+|| $Id: 406d06e6b76e3f88e01720a05f4b561d06940fc0 $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -134,6 +134,8 @@ struct udp_storage {
   struct fd_callback_box box;	/* Must be first. */
   int my_errno;
    
+  int inet_flags;
+
   int type;
   int protocol;
 
@@ -210,32 +212,56 @@ static void udp_bind(INT32 args)
     fd_close(fd);
   }
 
+  THIS->inet_flags &= ~PIKE_INET_FLAG_IPV6;
+
   addr_len = get_inet_addr(&addr, (args > 1 && Pike_sp[1-args].type==PIKE_T_STRING?
 				   Pike_sp[1-args].u.string->str : NULL),
 			   (Pike_sp[-args].type == PIKE_T_STRING?
 			    Pike_sp[-args].u.string->str : NULL),
 			   (Pike_sp[-args].type == PIKE_T_INT?
-			    Pike_sp[-args].u.integer : -1), 1);
+			    Pike_sp[-args].u.integer : -1),
+			   THIS->inet_flags);
+
+#ifdef AF_INET6
+  if (SOCKADDR_FAMILY(addr) == AF_INET6) {
+    THIS->inet_flags |= PIKE_INET_FLAG_IPV6;
+  }
+#endif
 
   fd = fd_socket(SOCKADDR_FAMILY(addr), THIS->type, THIS->protocol);
   if(fd < 0)
   {
     pop_n_elems(args);
     THIS->my_errno=errno;
-    Pike_error("Stdio.UDP->bind: failed to create socket\n");
+    Pike_error("Stdio.UDP->bind: failed to create socket (%d, %d, %d)\n",
+	       SOCKADDR_FAMILY(addr), THIS->type, THIS->protocol);
   }
 
   /* Make sure this fd gets closed on exec. */
   set_close_on_exec(fd, 1);
 
   o=1;
-  if(fd_setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *)&o, sizeof(int)) < 0)
+  /* linux kernel commit f24d43c07e208372aa3d3bff419afbf43ba87698 introduces
+   * a behaviour change where you can get used random ports if bound with
+   * SO_REUSEADDR. */
+  if(addr.ipv4.sin_port && fd_setsockopt(fd, SOL_SOCKET,SO_REUSEADDR,
+					 (char *)&o, sizeof(int)) < 0)
   {
     fd_close(fd);
     THIS->my_errno=errno;
     Pike_error("Stdio.UDP->bind: setsockopt SO_REUSEADDR failed\n");
   }
 
+#if defined(IPV6_V6ONLY) && defined(IPPROTO_IPV6)
+  if (SOCKADDR_FAMILY(addr) == AF_INET6) {
+    /* Attempt to enable dual-stack (ie mapped IPv4 adresses). Needed on WIN32.
+     * cf http://msdn.microsoft.com/en-us/library/windows/desktop/bb513665(v=vs.85).aspx
+     */
+    o = 0;
+    fd_setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&o, sizeof(int));
+  }
+#endif
+
 #ifndef SOL_IP
 #ifdef HAVE_GETPROTOBYNAME
   if (ip_proto_num == -1) {
@@ -328,8 +354,9 @@ void udp_enable_multicast(INT32 args)
 
   get_all_args("enable_multicast", args, "%s", &ip);
 
-  get_inet_addr(&reply, ip, NULL, -1, 1);
+  get_inet_addr(&reply, ip, NULL, -1, THIS->inet_flags);
 
+  /* FIXME: Implement support for IPv6! */
   if(SOCKADDR_FAMILY(reply) != AF_INET)
     Pike_error("Multicast only supported for IPv4.\n");
 
@@ -378,8 +405,9 @@ void udp_add_membership(INT32 args)
 
   get_all_args("add_membership", args, "%s.%s%d", &group, &address, &face);
 
-  get_inet_addr(&addr, group, NULL, -1, 1);
+  get_inet_addr(&addr, group, NULL, -1, THIS->inet_flags);
 
+  /* FIXME: Implement support for IPv6! */
   if(SOCKADDR_FAMILY(addr) != AF_INET)
     Pike_error("Multicast only supported for IPv4.\n");
 
@@ -388,8 +416,9 @@ void udp_add_membership(INT32 args)
   if( !address )
     sock.imr_interface.s_addr = htonl( INADDR_ANY );
   else {
-    get_inet_addr(&addr, address, NULL, -1, 1);
+    get_inet_addr(&addr, address, NULL, -1, THIS->inet_flags);
 
+    /* FIXME: Implement support for IPv6! */
     if(SOCKADDR_FAMILY(addr) != AF_INET)
       Pike_error("Multicast only supported for IPv4.\n");
 
@@ -419,8 +448,9 @@ void udp_drop_membership(INT32 args)
 
   get_all_args("drop_membership", args, "%s.%s%d", &group, &address, &face);
 
-  get_inet_addr(&addr, group, NULL, -1, 1);
+  get_inet_addr(&addr, group, NULL, -1, THIS->inet_flags);
 
+  /* FIXME: Implement support for IPv6! */
   if(SOCKADDR_FAMILY(addr) != AF_INET)
     Pike_error("Multicast only supported for IPv4.\n");
 
@@ -429,8 +459,9 @@ void udp_drop_membership(INT32 args)
   if( !address )
     sock.imr_interface.s_addr = htonl( INADDR_ANY );
   else {
-    get_inet_addr(&addr, address, NULL, -1, 1);
+    get_inet_addr(&addr, address, NULL, -1, THIS->inet_flags);
 
+    /* FIXME: Implement support for IPv6! */
     if(SOCKADDR_FAMILY(addr) != AF_INET)
       Pike_error("Multicast only supported for IPv4.\n");
 
@@ -648,8 +679,19 @@ void udp_read(INT32 args)
 
   push_constant_text("ip");
 #ifdef fd_inet_ntop
-  push_text( fd_inet_ntop( SOCKADDR_FAMILY(from), SOCKADDR_IN_ADDR(from),
-			   buffer, sizeof(buffer) ) );
+  fd_inet_ntop( SOCKADDR_FAMILY(from), SOCKADDR_IN_ADDR(from),
+		buffer, sizeof(buffer) );
+  /* NOTE: IPv6-mapped IPv4 addresses may only connect to other IPv4 addresses.
+   *
+   * Make the Pike-level code believe it has an actual IPv4 address
+   * when getting a mapped address (::FFFF:a.b.c.d).
+   */
+  if ((!strncmp(buffer, "::FFFF:", 7) || !strncmp(buffer, "::ffff:", 7)) &&
+      !strchr(buffer + 7, ':')) {
+    push_text(buffer+7);
+  } else {
+    push_text(buffer);
+  }
 #else
   push_text( inet_ntoa( *SOCKADDR_IN_ADDR(from) ) );
 #endif
@@ -708,7 +750,8 @@ void udp_sendto(INT32 args)
 			 (Pike_sp[1-args].type == PIKE_T_STRING?
 			  Pike_sp[1-args].u.string->str : NULL),
 			 (Pike_sp[1-args].type == PIKE_T_INT?
-			  Pike_sp[1-args].u.integer : -1), 1);
+			  Pike_sp[1-args].u.integer : -1),
+			 THIS->inet_flags);
 
   fd = FD;
   str = Pike_sp[2-args].u.string->str;
@@ -784,6 +827,7 @@ void zero_udp(struct object *o)
 {
   INIT_FD_CALLBACK_BOX(&THIS->box, NULL, o, -1, 0, got_udp_event);
   THIS->my_errno = 0;
+  THIS->inet_flags = PIKE_INET_FLAG_UDP;
   THIS->type=SOCK_DGRAM;
   THIS->protocol=0;
   /* map_variable handles read_callback. */
@@ -844,6 +888,7 @@ static void udp_set_nonblocking(INT32 args)
      pop_stack();
   }
   set_nonblocking(FD,1);
+  THIS->inet_flags |= PIKE_INET_FLAG_NB;
   ref_push_object(THISOBJ);
 }
 
@@ -855,6 +900,7 @@ static void udp_set_blocking(INT32 args)
 {
   if (FD < 0) Pike_error("File not open.\n");
   set_nonblocking(FD,0);
+  THIS->inet_flags &= ~PIKE_INET_FLAG_NB;
   pop_n_elems(args);
   ref_push_object(THISOBJ);
 }
@@ -897,7 +943,8 @@ static void udp_connect(INT32 args)
 			    (dest_port->type == PIKE_T_STRING?
 			     dest_port->u.string->str : NULL),
 			    (dest_port->type == PIKE_T_INT?
-			     dest_port->u.integer : -1), 0);
+			     dest_port->u.integer : -1),
+			    THIS->inet_flags);
 
   if(FD < 0)
   {
@@ -908,6 +955,17 @@ static void udp_connect(INT32 args)
 	Pike_error("Stdio.UDP->connect: failed to create socket\n");
      }
      set_close_on_exec(FD, 1);
+
+#if defined(IPV6_V6ONLY) && defined(IPPROTO_IPV6)
+     if (SOCKADDR_FAMILY(addr) == AF_INET6) {
+       /* Attempt to enable dual-stack (ie mapped IPv4 adresses).
+	* Needed on WIN32.
+	* cf http://msdn.microsoft.com/en-us/library/windows/desktop/bb513665(v=vs.85).aspx
+	*/
+       int o = 0;
+       fd_setsockopt(FD, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&o, sizeof(int));
+     }
+#endif
   }
 
   tmp=FD;
@@ -967,7 +1025,17 @@ static void udp_query_address(INT32 args)
 #endif
   sprintf(buffer+strlen(buffer)," %d",(int)(ntohs(addr.ipv4.sin_port)));
 
-  push_text(buffer);
+  /* NOTE: IPv6-mapped IPv4 addresses may only connect to other IPv4 addresses.
+   *
+   * Make the Pike-level code believe it has an actual IPv4 address
+   * when getting a mapped address (::FFFF:a.b.c.d).
+   */
+  if ((!strncmp(buffer, "::FFFF:", 7) || !strncmp(buffer, "::ffff:", 7)) &&
+      !strchr(buffer + 7, ':')) {
+    push_text(buffer+7);
+  } else {
+    push_text(buffer);
+  }
 }
 
 /*! @decl void set_backend (Pike.Backend backend)
