@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: passwords.c,v 1.47 2005/05/06 00:42:35 nilsson Exp $
+|| $Id: 643da396504c92875676dd53f69ab41edc3c0342 $
 */
 
 /*
@@ -219,7 +219,7 @@ void push_grent(struct group *ent)
   {
     char **cp = ent->gr_mem;
     int i=0;
-    while(cp[i]) push_text(cp[i++]);
+    while(cp && cp[i]) push_text(cp[i++]);
     f_aggregate(i);
   }
   f_aggregate(4);
