@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 8905ffa53035c0d0984e75716d8ef7cdc9de5cc7 $
+|| $Id: d1d2c5fdd465f2c2acdbfbdab0fa70edf2434f10 $
 */
 
 /*
@@ -172,7 +172,7 @@ HCHAR *NameNH(hubbe_search)(struct hubbe_searcher *s,
 
   NameH(HUBBE_ALIGN)(q);
 
-  for(;q<=end-sizeof(INT32);q+=max)
+  for(;q<=(HCHAR*)(((char *)end)-sizeof(INT32));q+=max)
   {
     h=tmp=NameH(GET_4_ALIGNED_CHARS)(q);
     
