@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 8ec30b55f7630bf5303666ccdcca8350b6787a01 $
+|| $Id: 5ec68be7ff6af0ad24902ee8647f2bf23955252b $
 */
 
 #ifndef PIKE_THREADLIB_H
@@ -610,12 +610,14 @@ static INLINE int threads_disabled_wait(void)
     _do_mt_unlock_interpreter();					\
   } while (0)
 #define co_wait_interpreter(COND) do {					\
+    int owner = threads_disabled;					\
     low_co_wait_interpreter(COND);					\
-    if (threads_disabled) threads_disabled_wait();			\
+    if (!owner && threads_disabled) threads_disabled_wait();		\
   } while (0)
 #define co_wait_interpreter_timeout(COND, SECS, NANOS) do {		\
+    int owner = threads_disabled;					\
     low_co_wait_interpreter_timeout(COND, SECS, NANOS);			\
-    if (threads_disabled) threads_disabled_wait();			\
+    if (!owner && threads_disabled) threads_disabled_wait();		\
   } while (0)
 
 #ifdef INTERNAL_PROFILING
