@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 25b7f86eb25bf8327c1b02fc28792eaa53e08aa4 $
+|| $Id: f413a4a7084c43441fe7ae5f2621eef5fac3efb5 $
 */
 
 #ifndef PORT_H
@@ -329,39 +329,53 @@ long long gethrtime(void);
 #define hrtime_t long long
 #endif
 
+#ifdef HAVE_UNION_INIT
+#define INFNAN_INDEX 0
+#define INFNAN_DATATYPE union
+#define INFNAN_INIT 
+#else
+#define INFNAN_INDEX -1
+#define INFNAN_DATATYPE struct
+#define INFNAN_INIT , {0.0}
+#endif
+
 #ifdef DOUBLE_IS_IEEE_BIG
-#define DECLARE_INF static const struct { unsigned char c[8]; double d[1]; } \
-	inf_ = { { 0x7f, 0xf0, 0, 0, 0, 0, 0, 0 }, { 0.0 } };
-#define DECLARE_NAN static const struct { unsigned char c[8]; double d[1]; } \
-	nan_ = { { 0x7f, 0xf8, 0, 0, 0, 0, 0, 0 }, { 0.0 } };
-#define MAKE_INF(s) ((s)*inf_.d[-1])
-#define MAKE_NAN() (nan_.d[-1])
+#define DECLARE_INF static const INFNAN_DATATYPE { unsigned char c[8]; double d[1]; } \
+	inf_ = { { 0x7f, 0xf0, 0, 0, 0, 0, 0, 0 } INFNAN_INIT };
+#define DECLARE_NAN static const INFNAN_DATATYPE { unsigned char c[8]; double d[1]; } \
+	nan_ = { { 0x7f, 0xf8, 0, 0, 0, 0, 0, 0 } INFNAN_INIT };
+#define MAKE_INF(s) ((s)*inf_.d[INFNAN_INDEX])
+#define MAKE_NAN() (nan_.d[INFNAN_INDEX])
 #else
 #ifdef DOUBLE_IS_IEEE_LITTLE
-#define DECLARE_INF static const struct { unsigned char c[8]; double d[1]; } \
-	inf_ = { { 0, 0, 0, 0, 0, 0, 0xf0, 0x7f }, { 0.0 } };
-#define DECLARE_NAN static const struct { unsigned char c[8]; double d[1]; } \
-	nan_ = { { 0, 0, 0, 0, 0, 0, 0xf8, 0x7f }, { 0.0 } };
-#define MAKE_INF(s) ((s)*inf_.d[-1])
-#define MAKE_NAN() (nan_.d[-1])
+#define DECLARE_INF static const INFNAN_DATATYPE { unsigned char c[8]; double d[1]; } \
+	inf_ = { { 0, 0, 0, 0, 0, 0, 0xf0, 0x7f } INFNAN_INIT };
+#define DECLARE_NAN static const INFNAN_DATATYPE { unsigned char c[8]; double d[1]; } \
+	nan_ = { { 0, 0, 0, 0, 0, 0, 0xf8, 0x7f } INFNAN_INIT };
+#define MAKE_INF(s) ((s)*inf_.d[INFNAN_INDEX])
+#define MAKE_NAN() (nan_.d[INFNAN_INDEX])
 #else
 #ifdef FLOAT_IS_IEEE_BIG
-#define DECLARE_INF static const struct { unsigned char c[4]; float f[1]; } \
-	inf_ = { { 0x7f, 0x80, 0, 0 }, { 0.0 } };
-#define DECLARE_NAN static const struct { unsigned char c[4]; float f[1]; } \
-	nan_ = { { 0x7f, 0xc0, 0, 0 }, { 0.0 } };
-#define MAKE_INF(s) ((s)*inf_.f[-1])
-#define MAKE_NAN() (nan_.f[-1])
+#define DECLARE_INF static const INFNAN_DATATYPE { unsigned char c[4]; float f[1]; } \
+	inf_ = { { 0x7f, 0x80, 0, 0 } INFNAN_INIT };
+#define DECLARE_NAN static const INFNAN_DATATYPE { unsigned char c[4]; float f[1]; } \
+	nan_ = { { 0x7f, 0xc0, 0, 0 } INFNAN_INIT };
+#define MAKE_INF(s) ((s)*inf_.f[INFNAN_INDEX])
+#define MAKE_NAN() (nan_.f[INFNAN_INDEX])
 #else
 #ifdef FLOAT_IS_IEEE_LITTLE
-#define DECLARE_INF static const struct { unsigned char c[4]; float f[1]; } \
-	inf_ = { { 0, 0, 0x80, 0x7f }, { 0.0 } };
-#define DECLARE_NAN static const struct { unsigned char c[4]; float f[1]; } \
-	nan_ = { { 0, 0, 0xc0, 0x7f }, { 0.0 } };
-#define MAKE_INF(s) ((s)*inf_.f[-1])
-#define MAKE_NAN() (nan_.f[-1])
+#define DECLARE_INF static const INFNAN_DATATYPE { unsigned char c[4]; float f[1]; } \
+	inf_ = { { 0, 0, 0x80, 0x7f } INFNAN_INIT };
+#define DECLARE_NAN static const INFNAN_DATATYPE { unsigned char c[4]; float f[1]; } \
+	nan_ = { { 0, 0, 0xc0, 0x7f } INFNAN_INIT };
+#define MAKE_INF(s) ((s)*inf_.f[INFNAN_INDEX])
+#define MAKE_NAN() (nan_.f[INFNAN_INDEX])
 #else
 
+#undef INFNAN_INDEX
+#undef INFNAN_DATATYPE
+#undef INFNAN_INIT 
+
 #define DECLARE_INF
 #define DECLARE_NAN
 
@@ -419,4 +433,12 @@ double LDEXP(double x, int exp);
 void _dosmaperr(int x);
 #endif
 
+#ifdef __clang__
+#define PIKE_CLANG_FEATURE(x)   __has_feature(x)
+#define PIKE_CLANG_BUILTIN(x)   __has_builtin(x)
+#else
+#define PIKE_CLANG_FEATURE(x)	(0)
+#define PIKE_CLANG_BUILTIN(x)   (0)
+#endif
+
 #endif
