@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: program.h,v 1.267 2009/09/12 13:31:38 grubba Exp $
+|| $Id: eacc5bf5c5eb33fc86d7e520964d651cf8ea9536 $
 */
 
 #ifndef PROGRAM_H
@@ -277,6 +277,7 @@ struct identifier
   unsigned INT16 opt_flags;		/* OPT_??? */
 #ifdef PROFILING
   unsigned INT32 num_calls;		/* Total number of calls. */
+  unsigned INT32 recur_depth;		/* Recursion depth during timing. */
   cpu_time_t total_time;		/* Total time with children. */
   cpu_time_t self_time;			/* Total time excluding children. */
 #endif /* PROFILING */
@@ -653,7 +654,6 @@ static INLINE int CHECK_IDREF_RANGE (int x, const struct program *p)
 #define PROG_FROM_INT(P,X) PROG_FROM_PTR(P, PTR_FROM_INT(P, X))
 #define ID_FROM_INT(P,X) ID_FROM_PTR(P, PTR_FROM_INT(P, X))
 
-#define FIND_LFUN(P,N) ( dmalloc_touch(struct program *,(P))->flags & PROGRAM_FIXED?((P)->lfuns[(N)]):low_find_lfun((P), (N)) )
 #define QUICK_FIND_LFUN(P,N) (dmalloc_touch(struct program *,(P))->lfuns[N])
 
 #ifdef DO_PIKE_CLEANUP
@@ -848,7 +848,7 @@ PMOD_EXPORT int simple_add_variable(const char *name,
 			const char *type,
 			INT32 flags);
 PMOD_EXPORT int add_constant(struct pike_string *name,
-		 struct svalue *c,
+		 const struct svalue *c,
 		 INT32 flags);
 PMOD_EXPORT int simple_add_constant(const char *name,
 			struct svalue *c,
@@ -888,6 +888,7 @@ int really_low_find_shared_string_identifier(struct pike_string *name,
 					     struct program *prog,
 					     int flags);
 PMOD_EXPORT int low_find_lfun(struct program *p, ptrdiff_t lfun);
+PMOD_EXPORT int find_lfun_fatal(struct program *p, ptrdiff_t lfun);
 int lfun_lookup_id(struct pike_string *lfun_name);
 int low_find_shared_string_identifier(struct pike_string *name,
 				      struct program *prog);
@@ -896,7 +897,7 @@ int find_shared_string_identifier(struct pike_string *name,
 				  struct program *prog);
 PMOD_EXPORT int find_identifier(const char *name,struct program *prog);
 int store_prog_string(struct pike_string *str);
-int store_constant(struct svalue *foo,
+int store_constant(const struct svalue *foo,
 		   int equal,
 		   struct pike_string *constant_name);
 struct array *program_indices(struct program *p);
@@ -947,6 +948,7 @@ void yytype_error(const char *msg, struct pike_type *expected_t,
 struct pike_string *format_exception_for_error_msg (struct svalue *thrown);
 void handle_compile_exception (const char *yyerror_fmt, ...);
 struct supporter_marker;
+void count_memory_in_supporter_markers(size_t *num, size_t *size);
 void verify_supporters(void);
 void init_supporter(struct Supporter *s,
 		    supporter_callback *fun,
@@ -1011,6 +1013,13 @@ void make_program_executable(struct program *p);
 /* Prototypes end here */
 
 void count_memory_in_programs(size_t *, size_t *);
+static INLINE int FIND_LFUN(struct program * p, int lfun) {
+    dmalloc_touch(struct program*, p);
+    if (lfun < 0 || lfun >= NUM_LFUNS) return find_lfun_fatal(p, lfun);
+
+    if (p->flags & PROGRAM_FIXED) return p->lfuns[lfun];
+    return low_find_lfun(p, lfun);
+}
 
 #ifndef PIKE_USE_MACHINE_CODE
 #define make_program_executable(X)
