@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: b9bba00755e7e288a4e0ac720bccc3a9d9b47134 $
+|| $Id: 60b64f406d7ef6454d543779199f8367833199e6 $
 */
 
 #include "global.h"
@@ -2586,7 +2586,7 @@ PMOD_EXPORT ptrdiff_t string_builder_quote_string(struct string_builder *buf,
     if (ch < 0 || ch > 0xffff) {
       /* Huge character. */
       string_builder_binary_strcat(buf, "\\U", 2);
-      string_builder_append_integer(buf, ch, 16, APPEND_ZERO_PAD, 8, 8);
+      string_builder_append_integer(buf, (unsigned INT32)ch, 16, APPEND_ZERO_PAD, 8, 8);
     } else if (ch > 0xff) {
       /* Unicode character. */
       string_builder_binary_strcat(buf, "\\u", 2);
@@ -2675,6 +2675,7 @@ PMOD_EXPORT void string_builder_append_integer(struct string_builder *s,
 
   tmp = val;
   if (base & (base - 1)) {
+    size_t cnt;
     /* Calculate the output length.
      * Use do-while to ensure that zero isn't output as an empty string.
      */
@@ -2688,39 +2689,43 @@ PMOD_EXPORT void string_builder_append_integer(struct string_builder *s,
     if (len < precision) len = precision;
 
     /* Perform padding. */
-    if ((len < min_width) && !(flags & APPEND_LEFT)) {
-      string_builder_fill(s, min_width - len, MKPCHARP("    ", 0),
-			  4, 0);
+    if (!(flags & APPEND_LEFT)) {
+      if (len < min_width) {
+	string_builder_fill(s, min_width - len, MKPCHARP("    ", 0),
+			    4, 0);
+      }
       min_width = 0;
     }
 
+    cnt = len;
+
     tmp = val;
     switch(s->s->size_shift) {
     case 0:
       {
 	p_wchar0 *p = string_builder_allocate(s, len, 0);
 	do {
-	  p[--len] = numbers[tmp%base];
+	  p[--cnt] = numbers[tmp%base];
 	  tmp /= base;
-	} while (len);
+	} while (cnt);
       }
       break;
     case 1:
       {
 	p_wchar1 *p = string_builder_allocate(s, len, 0);
 	do {
-	  p[--len] = numbers[tmp%base];
+	  p[--cnt] = numbers[tmp%base];
 	  tmp /= base;
-	} while (len);
+	} while (cnt);
       }
       break;
     case 2:
       {
 	p_wchar2 *p = string_builder_allocate(s, len, 0);
 	do {
-	  p[--len] = numbers[tmp%base];
+	  p[--cnt] = numbers[tmp%base];
 	  tmp /= base;
-	} while (len);
+	} while (cnt);
       }
       break;
     }
