@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 4470adbb1a585ab71da35918400d65a941f081f3 $
+|| $Id: 943a8c1c7fc2ac027905176948b70f7c5603dacf $
 */
 
 #ifndef STRALLOC_H
@@ -87,7 +87,11 @@ PMOD_EXPORT struct pike_string *debug_findstring(const struct pike_string *foo);
 #define is_same_string(X,Y) ((X)==(Y))
 
 /* NB: This intentionally only works for narrow strings. */
-#define string_has_null(X) (strlen((X)->str)!=(size_t)(X)->len)
+#ifdef USE_VALGRIND
+# define string_has_null(X) (strnlen((X)->str, (size_t)(X)->len)!=(size_t)(X)->len)
+#else
+# define string_has_null(X) (strlen((X)->str)!=(size_t)(X)->len)
+#endif
 
 #ifdef PIKE_DEBUG
 #define STR0(X) ((p_wchar0 *)debug_check_size_shift((X),0)->str)
