@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: svalue.c,v 1.254 2009/08/20 16:24:36 mast Exp $
+|| $Id: d80abc925ffad024aa3c42f4c1bb2d611a271fc6 $
 */
 
 #include "global.h"
@@ -30,14 +30,10 @@
 
 #define sp Pike_sp
 
-PMOD_EXPORT const struct svalue svalue_undefined =
+PMOD_EXPORT struct svalue svalue_undefined =
   SVALUE_INIT (T_INT, NUMBER_UNDEFINED, 0);
-PMOD_EXPORT const struct svalue svalue_int_zero = SVALUE_INIT_INT (0);
-#ifdef HAVE_UNION_INIT
-PMOD_EXPORT const struct svalue svalue_int_one = SVALUE_INIT_INT (1);
-#else
+PMOD_EXPORT struct svalue svalue_int_zero = SVALUE_INIT_INT (0);
 PMOD_EXPORT struct svalue svalue_int_one = SVALUE_INIT_INT (1);
-#endif
 
 #ifdef PIKE_DEBUG
 PMOD_EXPORT const char msg_type_error[] =
@@ -937,7 +933,7 @@ PMOD_EXPORT int low_short_is_equal(const union anything *a,
     sb.type=type;
   }else{
     sb.type=T_INT;
-    sa.u.integer=0;
+    sb.u.integer=0;
   }
 
   return low_is_equal(&sa,&sb,p);
@@ -1153,8 +1149,8 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
 PMOD_EXPORT int is_le(const struct svalue *a, const struct svalue *b)
 {
   /* Can't optimize this to !is_gt (a, b) since that'd assume total order. */
-  if (((1 << a->type) & (BIT_FUNCTION|BIT_PROGRAM|BIT_TYPE)) &&
-      ((1 << b->type) & (BIT_FUNCTION|BIT_PROGRAM|BIT_TYPE))) {
+  int mask = (1 << a->type)|(1 << b->type);
+  if (!(mask & ~(BIT_FUNCTION|BIT_PROGRAM|BIT_TYPE))) {
     /* NOTE: Special case for types, since is_eq() below only does
      * a pointer comparison.
      */
@@ -1194,6 +1190,25 @@ PMOD_EXPORT int is_le(const struct svalue *a, const struct svalue *b)
     free_type(a_type);
     free_type(b_type);
     return res;
+  } else if (mask == (BIT_INT | BIT_FLOAT)) {
+    /* Special case, since is_eq() won't promote ints to floats... */
+    FLOAT_TYPE aa, bb;
+    if (a->type == PIKE_T_INT) {
+      aa = (FLOAT_TYPE)a->u.integer;
+      bb = b->u.float_number;
+    } else {
+      aa = a->u.float_number;
+      bb = (FLOAT_TYPE)b->u.integer;
+    }
+    if (aa == bb) return 1;
+#ifdef HAVE_ISLESS
+    return isless(aa, bb);
+#else
+    if (PIKE_ISUNORDERED(aa, bb)) {
+      return 0;
+    }
+    return aa < bb;
+#endif
   }
   return is_lt (a, b) || is_eq (a, b);
 }
@@ -1243,7 +1258,7 @@ static int no_pike_calls = 0;	/* FIXME: Use TLS for this. */
  */
 PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct processing *p)
 {
-  char buf[50];
+  char buf[MAXIMUM (50, MAX_FLOAT_SPRINTF_LEN)];
 
   /* This needs to be a bit lower than LOW_C_STACK_MARGIN so that the
    * the raw error can be printed in exit_on_error. */
@@ -1252,6 +1267,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
   check_svalue_type (s);
   check_refs(s);
 
+  /* fprintf(stderr, "Describing svalue: %s\n", get_name_of_type(s->type)); */
+    
   indent+=2;
   switch(s->type)
   {
@@ -1714,30 +1731,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
     }
 
     case T_FLOAT:
-      {
-	FLOAT_TYPE d = s->u.float_number;
-	if (d != d) {
-	  my_strcat("nan");
-	} else if (d && (d+d == d)) {
-	  if (d > 0.0) {
-	    my_strcat("inf");
-	  } else {
-	    my_strcat("-inf");
-	  }
-	} else {
-	  sprintf(buf, "%.*"PRINTPIKEFLOAT"g", PIKEFLOAT_DIG, d);
-	  my_strcat(buf);
-	  if (!STRCHR (buf, '.') && !STRCHR (buf, ',') && !STRCHR (buf, 'e'))
-	    /* A small float number without fraction can be
-	     * indistinguishable from an integer when formatted by %g.
-	     * Kludge: Check for ',' too if someone has managed to
-	     * meddle with the LC_NUMERIC locale (not that the dot
-	     * below is really the right thing then, but at least it
-	     * doesn't become "47,11.0"). */
-	    /* Note: Code dup in operators.c:o_cast_to_string. */
-	    my_strcat (".0");
-	}
-      }
+      format_pike_float (buf, s->u.float_number);
+      my_strcat (buf);
       break;
 
     case T_ARRAY:
@@ -1850,6 +1845,7 @@ PMOD_EXPORT void print_short_svalue_compact (FILE *out, const union anything *a,
   else {
     struct svalue sval;
     sval.type = type;
+    sval.subtype = 0;
     sval.u = *a;
     print_svalue_compact (out, &sval);
   }
@@ -1896,49 +1892,50 @@ PMOD_EXPORT void copy_svalues_recursively_no_free(struct svalue *to,
 {
   ONERROR err;
   int allocated_here = 0;
-  if (!m) {
-    m = allocate_mapping(num);
-    allocated_here = 1;
-    SET_ONERROR(err, do_free_mapping, m);
-  }
   while(num--)
   {
     struct svalue *tmp;
-
+    int from_type = from->type;
+    
     check_svalue_type (from);
     check_refs(from);
-
-    if ((tmp = low_mapping_lookup(m, from))) {
-      *to = *tmp;
-      if (tmp->type <= MAX_REF_TYPE) add_ref(tmp->u.dummy);
-      to++;
-      from++;
-      continue;
+    
+    if (from_type == T_ARRAY ||
+	from_type == T_MAPPING ||
+	from_type == T_MULTISET) {
+      /* Recursive data */
+      if (m && (tmp = low_mapping_lookup(m, from))) {
+	*to = *tmp;
+	if (tmp->type <= MAX_REF_TYPE) add_ref(tmp->u.dummy);
+      } else {
+#define ALLOC_DUPL_MAPPING(type_hint)				\
+	do if (!m && (type_hint) & BIT_COMPLEX) {		\
+	    m = allocate_mapping(num && 1);			\
+	    allocated_here = 1;					\
+	    SET_ONERROR(err, do_free_mapping, m);		\
+	  } while (0)
+	  
+	if (from_type == T_ARRAY) {
+	  struct array *ar = from->u.array;
+	  ALLOC_DUPL_MAPPING(ar->type_field);
+	  to->u.array = copy_array_recursively(ar, m);
+	} else if (from_type == T_MAPPING) {
+	  struct mapping *ma = from->u.mapping;
+	  ALLOC_DUPL_MAPPING(m_ind_types(ma) | m_val_types(ma));
+	  to->u.mapping = copy_mapping_recursively(ma, m);
+	} else {
+	  struct multiset *mu = from->u.multiset;
+	  ALLOC_DUPL_MAPPING(multiset_ind_types(mu) | multiset_val_types(mu));
+	  to->u.multiset = copy_multiset_recursively(mu, m);
+	}
+	to->type = from_type;
+#undef ALLOC_DUPL_MAPPING
+      }
+    } else {
+      *to = *from;
+      if (from_type <= MAX_REF_TYPE) add_ref(from->u.array);
     }
-
-    switch(from->type)
-    {
-    default:
-      *to=*from;
-      if(from->type <= MAX_REF_TYPE) add_ref(from->u.array);
-      break;
-
-    case T_ARRAY:
-      to->u.array=copy_array_recursively(from->u.array,m);
-      to->type=T_ARRAY;
-      break;
-
-    case T_MAPPING:
-      to->u.mapping=copy_mapping_recursively(from->u.mapping,m);
-      to->type=T_MAPPING;
-      break;
-
-    case T_MULTISET:
-      to->u.multiset=copy_multiset_recursively(from->u.multiset,m);
-      to->type=T_MULTISET;
-      break;
-    }
-    mapping_insert(m, from, to);
+    
     to++;
     from++;
   }
@@ -2291,9 +2288,16 @@ void gc_check_weak_short_svalue(const union anything *u, TYPE_T type)
 
 #define GC_DONT_MARK(U, TN) do {} while (0)
 
+#define MARK_PRE {							\
+    DO_IF_DEBUG (							\
+      if (!s->u.refs)							\
+	(gc_fatal_2 (s->u.ptr, s->type, 0, "Marking thing without refs.\n")); \
+    );									\
+  }
+
 #define DO_MARK_FUNC_SVALUE(U, T, ZAP, GC_DO)				\
       if (s->subtype == FUNCTION_BUILTIN) {				\
-	DO_IF_DEBUG (if (d_flag) gc_mark (s->u.efun->name));		\
+	DO_IF_DEBUG (if (d_flag) gc_mark (s->u.efun->name, T_STRING));	\
 	DO_IF_DEBUG_OR_CLEANUP (GC_DO_MARK ((*s->u.efun), type));	\
 	break;								\
       }									\
@@ -2310,7 +2314,7 @@ void gc_check_weak_short_svalue(const union anything *u, TYPE_T type)
 	GC_DO_MARK(U, TN)
 
 #define DO_MARK_STRING(U)						\
-      DO_IF_DEBUG(if (U.refs && d_flag) gc_mark(U.string))
+      DO_IF_DEBUG(if (U.refs && d_flag) gc_mark(U.string, T_STRING))
 
 #define DONT_MARK_STRING(U)
 
@@ -2323,7 +2327,7 @@ PMOD_EXPORT TYPE_FIELD real_gc_mark_svalues(struct svalue *s, size_t num)
   {
     dmalloc_touch_svalue(s);
     GC_RECURSE_SWITCH((s->u), (s->type), ZAP_SVALUE, DONT_FREE_WEAK,
-		      GC_DO_MARK, {},
+		      GC_DO_MARK, MARK_PRE,
 		      DO_MARK_FUNC_SVALUE, GC_DO_MARK,
 		      DO_MARK_STRING, GC_DO_MARK);
     t |= 1 << s->type;
@@ -2340,7 +2344,7 @@ TYPE_FIELD gc_mark_weak_svalues(struct svalue *s, size_t num)
   {
     dmalloc_touch_svalue(s);
     GC_RECURSE_SWITCH((s->u), (s->type), ZAP_SVALUE, FREE_WEAK,
-		      GC_DONT_MARK, {},
+		      GC_DONT_MARK, MARK_PRE,
 		      DO_MARK_FUNC_SVALUE, DO_MARK_OBJ_WEAK,
 		      DO_MARK_STRING, GC_DO_MARK);
     t |= 1 << s->type;
@@ -2375,7 +2379,7 @@ int gc_mark_without_recurse(struct svalue *s)
   int freed = 0;
   dmalloc_touch_svalue(s);
   GC_RECURSE_SWITCH((s->u), (s->type), ZAP_SVALUE, DONT_FREE_WEAK,
-		    GC_DONT_MARK, {},
+		    GC_DONT_MARK, MARK_PRE,
 		    DONT_MARK_FUNC_SVALUE, GC_DONT_MARK,
 		    DONT_MARK_STRING, GC_DONT_MARK);
   return freed;
@@ -2386,7 +2390,7 @@ int gc_mark_weak_without_recurse(struct svalue *s)
   int freed = 0;
   dmalloc_touch_svalue(s);
   GC_RECURSE_SWITCH((s->u), (s->type), ZAP_SVALUE, FREE_WEAK,
-		    GC_DONT_MARK, {},
+		    GC_DONT_MARK, MARK_PRE,
 		    DONT_MARK_FUNC_SVALUE, GC_DONT_MARK,
 		    DONT_MARK_STRING, GC_DONT_MARK);
   return freed;
@@ -2532,7 +2536,7 @@ PMOD_EXPORT INT32 pike_sizeof(const struct svalue *s)
   }
 }
 
-int svalues_are_constant(struct svalue *s,
+int svalues_are_constant(const struct svalue *s,
 			 INT32 num,
 			 TYPE_FIELD hint,
 			 struct processing *p)
