@@ -1,6 +1,7 @@
 /* Shared speed subroutines.
 
-Copyright 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation,
+Inc.
 
 This file is part of the GNU MP Library.
 
@@ -16,8 +17,10 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+MA 02110-1301, USA. */
+
+#define __GMP_NO_ATTRIBUTE_CONST_PURE
 
 #include <errno.h>
 #include <fcntl.h>
@@ -124,12 +127,14 @@ speed_measure (double (*fun) _PROTO ((struct speed_params *s)),
                struct speed_params *s)
 {
 #define TOLERANCE    1.005  /* 0.5% */
+  const int max_zeros = 10;
 
   struct speed_params  s_dummy;
   int     i, j, e;
   double  t[30];
   double  t_unsorted[30];
   double  reps_d;
+  int     zeros = 0;
 
   /* Use dummy parameters if caller doesn't provide any.  Only a few special
      "fun"s will cope with this, speed_noop() is one.  */
@@ -138,7 +143,7 @@ speed_measure (double (*fun) _PROTO ((struct speed_params *s)),
       memset (&s_dummy, '\0', sizeof (s_dummy));
       s = &s_dummy;
     }
-  
+
   s->reps = 1;
   s->time_divisor = 1.0;
   for (i = 0; i < numberof (t); i++)
@@ -151,8 +156,19 @@ speed_measure (double (*fun) _PROTO ((struct speed_params *s)),
           t[i] = (*fun) (s);
 
           if (speed_option_verbose >= 3)
-            printf("size=%ld reps=%u r=%ld attempt=%d  %.9f\n", 
-                   s->size, s->reps, s->r, i, t[i]);
+            gmp_printf("size=%ld reps=%u r=%Md attempt=%d  %.9f\n",
+                       (long) s->size, s->reps, s->r, i, t[i]);
+
+          if (t[i] == 0.0)
+            {
+              zeros++;
+              if (zeros > max_zeros)
+                {
+                  fprintf (stderr, "Fatal error: too many (%d) failed measurements (0.0)\n", zeros);
+                  abort ();
+                }
+              continue;
+            }
 
           if (t[i] == -1.0)
             return -1.0;
@@ -194,7 +210,7 @@ speed_measure (double (*fun) _PROTO ((struct speed_params *s)),
             if (t[j] <= t[j-e+1] * TOLERANCE)
               return t[j-e+1] / s->time_divisor;
         }
-    } 
+    }
 
   fprintf (stderr, "speed_measure() could not get %d results within %.1f%%\n",
            e, (TOLERANCE-1.0)*100.0);
@@ -209,7 +225,7 @@ speed_measure (double (*fun) _PROTO ((struct speed_params *s)),
 
 
 /* Read all of ptr,size to get it into the CPU memory cache.
-  
+
    A call to mpn_cache_fill_dummy() is used to make sure the compiler
    doesn't optimize away the whole loop.  Using "volatile mp_limb_t sum"
    would work too, but the function call means we don't rely on every
@@ -295,7 +311,7 @@ speed_cache_fill (struct speed_params *s)
       for (i = 0; i < s->src_num; i++)
         different |= (s->src[i].ptr != prev.src[i].ptr);
 
-      if (different) 
+      if (different)
         {
           if (s->dst_num != 0)
             {
@@ -333,26 +349,6 @@ speed_cache_fill (struct speed_params *s)
 }
 
 
-/* Adjust ptr to align to CACHE_LINE_SIZE bytes plus "align" limbs.  ptr
-   needs to have room for up to CACHE_LINE_SIZE-4 extra bytes.  */
-
-mp_ptr
-speed_tmp_alloc_adjust (void *ptr, mp_size_t align)
-{
-  /*
-  printf("%p %ld -> %p %X %X\n", ptr, align,
-         (mp_ptr) ptr 
-         + ((align - ((mp_size_t) ptr >> 2)) &
-            SPEED_TMP_ALLOC_ADJUST_MASK),
-         ((mp_size_t) ptr >> 2) & SPEED_TMP_ALLOC_ADJUST_MASK,
-         SPEED_TMP_ALLOC_ADJUST_MASK);
-  */
-
-  return (mp_ptr) ptr 
-    + ((align - ((mp_size_t) ptr >> 2)) & SPEED_TMP_ALLOC_ADJUST_MASK);
-}
-
-
 /* Miscellanous options accepted by tune and speed programs under -o. */
 
 void
@@ -372,7 +368,7 @@ speed_option_set (const char *s)
     {
       speed_option_verbose = n;
     }
-  else 
+  else
     {
       printf ("Unrecognised -o option: %s\n", s);
       exit (1);
@@ -475,6 +471,55 @@ speed_mpn_submul_1 (struct speed_params *s)
   SPEED_ROUTINE_MPN_UNARY_1 (mpn_submul_1);
 }
 
+#if HAVE_NATIVE_mpn_addmul_2
+double
+speed_mpn_addmul_2 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_2 (mpn_addmul_2);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_3
+double
+speed_mpn_addmul_3 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_3 (mpn_addmul_3);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_4
+double
+speed_mpn_addmul_4 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_4 (mpn_addmul_4);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_5
+double
+speed_mpn_addmul_5 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_5 (mpn_addmul_5);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_6
+double
+speed_mpn_addmul_6 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_6 (mpn_addmul_6);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_7
+double
+speed_mpn_addmul_7 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_7 (mpn_addmul_7);
+}
+#endif
+#if HAVE_NATIVE_mpn_addmul_8
+double
+speed_mpn_addmul_8 (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_UNARY_8 (mpn_addmul_8);
+}
+#endif
 
 double
 speed_mpn_mul_1 (struct speed_params *s)
@@ -491,7 +536,7 @@ speed_mpn_mul_1_inplace (struct speed_params *s)
 double
 speed_mpn_mul_2 (struct speed_params *s)
 {
-  SPEED_ROUTINE_MPN_MUL_2 (mpn_mul_2);
+  SPEED_ROUTINE_MPN_UNARY_2 (mpn_mul_2);
 }
 #endif
 
@@ -576,11 +621,13 @@ speed_mpn_preinv_divrem_1f (struct speed_params *s)
   SPEED_ROUTINE_MPN_PREINV_DIVREM_1F (mpn_preinv_divrem_1);
 }
 
+#if GMP_NUMB_BITS % 4 == 0
 double
 speed_mpn_mod_34lsub1 (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_MOD_34LSUB1 (mpn_mod_34lsub1);
 }
+#endif
 
 double
 speed_mpn_divrem_2 (struct speed_params *s)
@@ -720,6 +767,34 @@ speed_mpn_sub_n (struct speed_params *s)
 SPEED_ROUTINE_MPN_BINARY_N (mpn_sub_n);
 }
 
+#if HAVE_NATIVE_mpn_addlsh1_n
+double
+speed_mpn_addlsh1_n (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_BINARY_N (mpn_addlsh1_n);
+}
+#endif
+#if HAVE_NATIVE_mpn_sublsh1_n
+double
+speed_mpn_sublsh1_n (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_BINARY_N (mpn_sublsh1_n);
+}
+#endif
+#if HAVE_NATIVE_mpn_rsh1add_n
+double
+speed_mpn_rsh1add_n (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_BINARY_N (mpn_rsh1add_n);
+}
+#endif
+#if HAVE_NATIVE_mpn_rsh1sub_n
+double
+speed_mpn_rsh1sub_n (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_BINARY_N (mpn_rsh1sub_n);
+}
+#endif
 
 /* mpn_and_n etc can be macros and so have to be handled with
    SPEED_ROUTINE_MPN_BINARY_N_CALL forms */
@@ -823,27 +898,6 @@ speed_mpn_toom3_sqr_n (struct speed_params *s)
   SPEED_ROUTINE_MPN_TOOM3_SQR_N (mpn_toom3_sqr_n);
 }
 
-double
-speed_mpn_toom3_mul_n_mpn (struct speed_params *s)
-{
-  SPEED_ROUTINE_MPN_TOOM3_MUL_N (mpn_toom3_mul_n_mpn);
-}
-double
-speed_mpn_toom3_mul_n_open (struct speed_params *s)
-{
-  SPEED_ROUTINE_MPN_TOOM3_MUL_N (mpn_toom3_mul_n_open);
-}
-double
-speed_mpn_toom3_sqr_n_mpn (struct speed_params *s)
-{
-  SPEED_ROUTINE_MPN_TOOM3_SQR_N (mpn_toom3_sqr_n_mpn);
-}
-double
-speed_mpn_toom3_sqr_n_open (struct speed_params *s)
-{
-  SPEED_ROUTINE_MPN_TOOM3_SQR_N (mpn_toom3_sqr_n_open);
-}
-
 double
 speed_mpn_mul_fft_full (struct speed_params *s)
 {
@@ -869,7 +923,7 @@ speed_mpn_mul_fft_full_sqr (struct speed_params *s)
     int        k;                                       \
     unsigned   i;                                       \
     double     t;                                       \
-    TMP_DECL (marker);                                  \
+    TMP_DECL;                                  \
                                                         \
     SPEED_RESTRICT_COND (s->size >= 1);                 \
                                                         \
@@ -878,9 +932,9 @@ speed_mpn_mul_fft_full_sqr (struct speed_params *s)
     else                                                \
       k = mpn_fft_best_k (s->size, sqr);                \
                                                         \
-    TMP_MARK (marker);                                  \
+    TMP_MARK;                                  \
     pl = mpn_fft_next_size (s->size, k);                \
-    wp = SPEED_TMP_ALLOC_LIMBS (pl+1, s->align_wp);     \
+    SPEED_TMP_ALLOC_LIMBS (wp, pl+1, s->align_wp);      \
                                                         \
     speed_operand_src (s, s->xp, s->size);              \
     if (!sqr)                                           \
@@ -895,9 +949,9 @@ speed_mpn_mul_fft_full_sqr (struct speed_params *s)
     while (--i != 0);                                   \
     t = speed_endtime ();                               \
                                                         \
-    TMP_FREE (marker);                                  \
+    TMP_FREE;                                  \
     return t;                                           \
-  }  
+  }
 
 double
 speed_mpn_mul_fft (struct speed_params *s)
@@ -913,6 +967,16 @@ speed_mpn_mul_fft_sqr (struct speed_params *s)
     (mpn_mul_fft (wp, pl, s->xp, s->size, s->xp, s->size, k), 1);
 }
 
+double
+speed_mpn_mullow_n (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_MULLOW_N (mpn_mullow_n);
+}
+double
+speed_mpn_mullow_basecase (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_MULLOW_BASECASE (mpn_mullow_basecase);
+}
 
 double
 speed_mpn_gcd (struct speed_params *s)
@@ -930,7 +994,7 @@ double
 speed_mpn_gcd_finda (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_GCD_FINDA (mpn_gcd_finda);
-}  
+}
 #endif
 
 
@@ -939,6 +1003,13 @@ speed_mpn_gcdext (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_GCDEXT (mpn_gcdext);
 }
+#if 0
+double
+speed_mpn_gcdext_lehmer (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_GCDEXT (__gmpn_gcdext_lehmer);
+}
+#endif
 double
 speed_mpn_gcdext_single (struct speed_params *s)
 {
@@ -1004,6 +1075,12 @@ speed_mpn_sqrtrem (struct speed_params *s)
   SPEED_ROUTINE_MPN_SQRTREM (mpn_sqrtrem);
 }
 
+double
+speed_mpn_rootrem (struct speed_params *s)
+{
+  SPEED_ROUTINE_MPN_ROOTREM (mpn_rootrem);
+}
+
 
 double
 speed_mpz_fac_ui (struct speed_params *s)
@@ -1072,14 +1149,14 @@ double
 speed_noop (struct speed_params *s)
 {
   unsigned  i;
-  
+
   speed_starttime ();
   i = s->reps;
-  do 
+  do
     noop ();
   while (--i != 0);
   return speed_endtime ();
-}  
+}
 
 double
 speed_noop_wxs (struct speed_params *s)
@@ -1087,9 +1164,9 @@ speed_noop_wxs (struct speed_params *s)
   mp_ptr   wp;
   unsigned i;
   double   t;
-  TMP_DECL (marker);
+  TMP_DECL;
 
-  TMP_MARK (marker);
+  TMP_MARK;
   wp = TMP_ALLOC_LIMBS (1);
 
   speed_starttime ();
@@ -1099,9 +1176,9 @@ speed_noop_wxs (struct speed_params *s)
   while (--i != 0);
   t = speed_endtime ();
 
-  TMP_FREE (marker);
+  TMP_FREE;
   return t;
-}  
+}
 
 double
 speed_noop_wxys (struct speed_params *s)
@@ -1109,9 +1186,9 @@ speed_noop_wxys (struct speed_params *s)
   mp_ptr   wp;
   unsigned i;
   double   t;
-  TMP_DECL (marker);
+  TMP_DECL;
 
-  TMP_MARK (marker);
+  TMP_MARK;
   wp = TMP_ALLOC_LIMBS (1);
 
   speed_starttime ();
@@ -1121,9 +1198,9 @@ speed_noop_wxys (struct speed_params *s)
   while (--i != 0);
   t = speed_endtime ();
 
-  TMP_FREE (marker);
+  TMP_FREE;
   return t;
-}  
+}
 
 
 #define SPEED_ROUTINE_ALLOC_FREE(variables, calls)      \
@@ -1154,7 +1231,7 @@ speed_malloc_free (struct speed_params *s)
   SPEED_ROUTINE_ALLOC_FREE (void *p,
                             p = malloc (bytes);
                             free (p));
-}  
+}
 
 double
 speed_malloc_realloc_free (struct speed_params *s)
@@ -1164,7 +1241,7 @@ speed_malloc_realloc_free (struct speed_params *s)
                             p = malloc (BYTES_PER_MP_LIMB);
                             p = realloc (p, bytes);
                             free (p));
-}  
+}
 
 double
 speed_gmp_allocate_free (struct speed_params *s)
@@ -1173,7 +1250,7 @@ speed_gmp_allocate_free (struct speed_params *s)
   SPEED_ROUTINE_ALLOC_FREE (void *p,
                             p = (*__gmp_allocate_func) (bytes);
                             (*__gmp_free_func) (p, bytes));
-}  
+}
 
 double
 speed_gmp_allocate_reallocate_free (struct speed_params *s)
@@ -1184,7 +1261,7 @@ speed_gmp_allocate_reallocate_free (struct speed_params *s)
      p = (*__gmp_allocate_func) (BYTES_PER_MP_LIMB);
      p = (*__gmp_reallocate_func) (p, bytes, BYTES_PER_MP_LIMB);
      (*__gmp_free_func) (p, bytes));
-}  
+}
 
 double
 speed_mpz_init_clear (struct speed_params *s)
@@ -1192,7 +1269,7 @@ speed_mpz_init_clear (struct speed_params *s)
   SPEED_ROUTINE_ALLOC_FREE (mpz_t z,
                             mpz_init (z);
                             mpz_clear (z));
-}  
+}
 
 double
 speed_mpz_init_realloc_clear (struct speed_params *s)
@@ -1201,7 +1278,7 @@ speed_mpz_init_realloc_clear (struct speed_params *s)
                             mpz_init (z);
                             _mpz_realloc (z, s->size);
                             mpz_clear (z));
-}  
+}
 
 double
 speed_mpq_init_clear (struct speed_params *s)
@@ -1209,7 +1286,7 @@ speed_mpq_init_clear (struct speed_params *s)
   SPEED_ROUTINE_ALLOC_FREE (mpq_t q,
                             mpq_init (q);
                             mpq_clear (q));
-}  
+}
 
 double
 speed_mpf_init_clear (struct speed_params *s)
@@ -1217,7 +1294,7 @@ speed_mpf_init_clear (struct speed_params *s)
   SPEED_ROUTINE_ALLOC_FREE (mpf_t f,
                             mpf_init (f);
                             mpf_clear (f));
-}  
+}
 
 
 /* Compare this to mpn_add_n to see how much overhead mpz_add adds.  Note
@@ -1241,7 +1318,7 @@ speed_mpz_add (struct speed_params *s)
 
   speed_starttime ();
   i = s->reps;
-  do 
+  do
     {
       mpz_add (w, x, y);
     }
@@ -1252,7 +1329,7 @@ speed_mpz_add (struct speed_params *s)
   mpz_clear (x);
   mpz_clear (y);
   return t;
-}  
+}
 
 
 /* If r==0, calculate (size,size/2),
@@ -1274,7 +1351,7 @@ speed_mpz_bin_uiui (struct speed_params *s)
 
   speed_starttime ();
   i = s->reps;
-  do 
+  do
     {
       mpz_bin_uiui (w, s->size, k);
     }
@@ -1283,7 +1360,7 @@ speed_mpz_bin_uiui (struct speed_params *s)
 
   mpz_clear (w);
   return t;
-}  
+}
 
 
 /* The multiplies are successively dependent so the latency is measured, not
@@ -1385,41 +1462,68 @@ speed_umul_ppmm (struct speed_params *s)
 
 
 #if HAVE_NATIVE_mpn_umul_ppmm
-
-#if defined (__hppa) && W_TYPE_SIZE == 64
-#define CALL_MPN_UMUL_PPMM  (h = __MPN (umul_ppmm) (h, l, &l))
-#else
-#define CALL_MPN_UMUL_PPMM  (h = __MPN (umul_ppmm) (&l, h, l))
-#endif
-
 double
 speed_mpn_umul_ppmm (struct speed_params *s)
 {
   SPEED_MACRO_UMUL_PPMM_A;
   {
-    CALL_MPN_UMUL_PPMM;  h ^= s->xp_block[0]; l ^= s->yp_block[0];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[1]; l ^= s->yp_block[1];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[2]; l ^= s->yp_block[2];
-    CALL_MPN_UMUL_PPMM;  h ^= s->xp_block[3]; l ^= s->yp_block[3];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[4]; l ^= s->yp_block[4];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[5]; l ^= s->yp_block[5];
-    CALL_MPN_UMUL_PPMM;  h ^= s->xp_block[6]; l ^= s->yp_block[6];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[7]; l ^= s->yp_block[7];
-     CALL_MPN_UMUL_PPMM; h ^= s->xp_block[8]; l ^= s->yp_block[8];
-    CALL_MPN_UMUL_PPMM;  h ^= s->xp_block[9]; l ^= s->yp_block[9];
+    h = mpn_umul_ppmm (&l, h, l);  h ^= s->xp_block[0]; l ^= s->yp_block[0];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[1]; l ^= s->yp_block[1];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[2]; l ^= s->yp_block[2];
+    h = mpn_umul_ppmm (&l, h, l);  h ^= s->xp_block[3]; l ^= s->yp_block[3];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[4]; l ^= s->yp_block[4];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[5]; l ^= s->yp_block[5];
+    h = mpn_umul_ppmm (&l, h, l);  h ^= s->xp_block[6]; l ^= s->yp_block[6];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[7]; l ^= s->yp_block[7];
+     h = mpn_umul_ppmm (&l, h, l); h ^= s->xp_block[8]; l ^= s->yp_block[8];
+    h = mpn_umul_ppmm (&l, h, l);  h ^= s->xp_block[9]; l ^= s->yp_block[9];
   }
   SPEED_MACRO_UMUL_PPMM_B;
   {
-    CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-    CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-    CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-     CALL_MPN_UMUL_PPMM;
-    CALL_MPN_UMUL_PPMM;
+    h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+    h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+    h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+     h = mpn_umul_ppmm (&l, h, l);
+    h = mpn_umul_ppmm (&l, h, l);
+  }
+  SPEED_MACRO_UMUL_PPMM_C;
+}
+#endif
+
+#if HAVE_NATIVE_mpn_umul_ppmm_r
+double
+speed_mpn_umul_ppmm_r (struct speed_params *s)
+{
+  SPEED_MACRO_UMUL_PPMM_A;
+  {
+    h = mpn_umul_ppmm_r (h, l, &l);  h ^= s->xp_block[0]; l ^= s->yp_block[0];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[1]; l ^= s->yp_block[1];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[2]; l ^= s->yp_block[2];
+    h = mpn_umul_ppmm_r (h, l, &l);  h ^= s->xp_block[3]; l ^= s->yp_block[3];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[4]; l ^= s->yp_block[4];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[5]; l ^= s->yp_block[5];
+    h = mpn_umul_ppmm_r (h, l, &l);  h ^= s->xp_block[6]; l ^= s->yp_block[6];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[7]; l ^= s->yp_block[7];
+     h = mpn_umul_ppmm_r (h, l, &l); h ^= s->xp_block[8]; l ^= s->yp_block[8];
+    h = mpn_umul_ppmm_r (h, l, &l);  h ^= s->xp_block[9]; l ^= s->yp_block[9];
+  }
+  SPEED_MACRO_UMUL_PPMM_B;
+  {
+    h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+    h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+    h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+     h = mpn_umul_ppmm_r (h, l, &l);
+    h = mpn_umul_ppmm_r (h, l, &l);
   }
   SPEED_MACRO_UMUL_PPMM_C;
 }
@@ -1511,39 +1615,39 @@ speed_udiv_qrnnd (struct speed_params *s)
 }
 
 double
-speed_udiv_qrnnd_preinv (struct speed_params *s)
+speed_udiv_qrnnd_preinv1 (struct speed_params *s)
 {
   SPEED_ROUTINE_UDIV_QRNND_A (1);
   {
-    udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv1 (q, r, r, q, d, dinv);
   }
   SPEED_ROUTINE_UDIV_QRNND_B;
-}  
+}
 
 double
-speed_udiv_qrnnd_preinv2norm (struct speed_params *s)
+speed_udiv_qrnnd_preinv2 (struct speed_params *s)
 {
   SPEED_ROUTINE_UDIV_QRNND_A (1);
   {
-    udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-     udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
-    udiv_qrnnd_preinv2norm (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+     udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
+    udiv_qrnnd_preinv2 (q, r, r, q, d, dinv);
   }
   SPEED_ROUTINE_UDIV_QRNND_B;
 }
@@ -1565,31 +1669,45 @@ speed_udiv_qrnnd_c (struct speed_params *s)
     __udiv_qrnnd_c (q, r, r, q, d);
   }
   SPEED_ROUTINE_UDIV_QRNND_B;
-}  
+}
 
 #if HAVE_NATIVE_mpn_udiv_qrnnd
-
-#if defined (__hppa) && W_TYPE_SIZE == 64
-#define CALL_MPN_UDIV_QRNND  (q = __MPN (udiv_qrnnd) (r, q, d, &r))
-#else
-#define CALL_MPN_UDIV_QRNND  (q = __MPN (udiv_qrnnd) (&r, r, q, d))
-#endif
-
 double
 speed_mpn_udiv_qrnnd (struct speed_params *s)
 {
   SPEED_ROUTINE_UDIV_QRNND_A (1);
   {
-    CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-    CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-    CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-     CALL_MPN_UDIV_QRNND;
-    CALL_MPN_UDIV_QRNND;
+    q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+    q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+    q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+     q = mpn_udiv_qrnnd (&r, r, q, d);
+    q = mpn_udiv_qrnnd (&r, r, q, d);
+  }
+  SPEED_ROUTINE_UDIV_QRNND_B;
+}
+#endif
+
+#if HAVE_NATIVE_mpn_udiv_qrnnd_r
+double
+speed_mpn_udiv_qrnnd_r (struct speed_params *s)
+{
+  SPEED_ROUTINE_UDIV_QRNND_A (1);
+  {
+    q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+    q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+    q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+     q = mpn_udiv_qrnnd_r (r, q, d, &r);
+    q = mpn_udiv_qrnnd_r (r, q, d, &r);
   }
   SPEED_ROUTINE_UDIV_QRNND_B;
 }
@@ -1773,13 +1891,13 @@ double
 speed_mpn_get_str (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_GET_STR (mpn_get_str);
-}  
+}
 
 double
 speed_mpn_set_str (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_SET_STR (mpn_set_str);
-}  
+}
 double
 speed_mpn_set_str_basecase (struct speed_params *s)
 {
@@ -1796,4 +1914,107 @@ double
 speed_MPN_ZERO (struct speed_params *s)
 {
   SPEED_ROUTINE_MPN_ZERO_CALL (MPN_ZERO (wp, s->size));
-}  
+}
+
+
+int
+speed_randinit (struct speed_params *s, gmp_randstate_ptr rstate)
+{
+  if (s->r == 0)
+    gmp_randinit_default (rstate);
+  else if (s->r == 1)
+    gmp_randinit_mt (rstate);
+  else
+    {
+      return gmp_randinit_lc_2exp_size (rstate, s->r);
+    }
+  return 1;
+}
+
+double
+speed_gmp_randseed (struct speed_params *s)
+{
+  gmp_randstate_t  rstate;
+  unsigned  i;
+  double    t;
+  mpz_t     x;
+
+  SPEED_RESTRICT_COND (s->size >= 1);
+  SPEED_RESTRICT_COND (speed_randinit (s, rstate));
+
+  /* s->size bits of seed */
+  mpz_init_set_n (x, s->xp, s->size);
+  mpz_fdiv_r_2exp (x, x, (unsigned long) s->size);
+
+  /* cache priming */
+  gmp_randseed (rstate, x);
+
+  speed_starttime ();
+  i = s->reps;
+  do
+    gmp_randseed (rstate, x);
+  while (--i != 0);
+  t = speed_endtime ();
+
+  gmp_randclear (rstate);
+  mpz_clear (x);
+  return t;
+}
+
+double
+speed_gmp_randseed_ui (struct speed_params *s)
+{
+  gmp_randstate_t  rstate;
+  unsigned  i, j;
+  double    t;
+
+  SPEED_RESTRICT_COND (speed_randinit (s, rstate));
+
+  /* cache priming */
+  gmp_randseed_ui (rstate, 123L);
+
+  speed_starttime ();
+  i = s->reps;
+  j = 0;
+  do
+    {
+      gmp_randseed_ui (rstate, (unsigned long) s->xp_block[j]);
+      j++;
+      if (j >= SPEED_BLOCK_SIZE)
+        j = 0;
+    }
+  while (--i != 0);
+  t = speed_endtime ();
+
+  gmp_randclear (rstate);
+  return t;
+}
+
+double
+speed_mpz_urandomb (struct speed_params *s)
+{
+  gmp_randstate_t  rstate;
+  mpz_t     z;
+  unsigned  i;
+  double    t;
+
+  SPEED_RESTRICT_COND (s->size >= 0);
+  SPEED_RESTRICT_COND (speed_randinit (s, rstate));
+
+  mpz_init (z);
+
+  /* cache priming */
+  mpz_urandomb (z, rstate, (unsigned long) s->size);
+  mpz_urandomb (z, rstate, (unsigned long) s->size);
+
+  speed_starttime ();
+  i = s->reps;
+  do
+    mpz_urandomb (z, rstate, (unsigned long) s->size);
+  while (--i != 0);
+  t = speed_endtime ();
+
+  mpz_clear (z);
+  gmp_randclear (rstate);
+  return t;
+}
